@@ -291,8 +291,9 @@ static pj_status_t ca_factory_init(pjmedia_aud_dev_factory *f)
         cdi = &cf->dev_info[i];
         pj_bzero(cdi, sizeof(*cdi));
         cdi->dev_id = 0;
-        strcpy(cdi->info.name, "iPhone IO device");
-        strcpy(cdi->info.driver, "apple");
+        pj_ansi_strxcpy(cdi->info.name, "iPhone IO device",
+                        sizeof(cdi->info.name));
+        pj_ansi_strxcpy(cdi->info.driver, "apple", sizeof(cdi->info.driver));
         cdi->info.input_count = 1;
         cdi->info.output_count = 1;
         cdi->info.default_samples_per_sec = 8000;
@@ -549,7 +550,8 @@ static pj_status_t ca_factory_refresh(pjmedia_aud_dev_factory *f)
                                    0, NULL,
                                    &size, (void *)cdi->info.name);
 
-        strcpy(cdi->info.driver, "core audio");
+        pj_ansi_strxcpy(cdi->info.driver, "core audio",
+                        sizeof(cdi->info.driver));
 
         /* Get the number of input channels */
         addr.mSelector = kAudioDevicePropertyStreamConfiguration;
