@@ -93,13 +93,19 @@ static const char *desc =
  "  --srtp-dtls-server    Use DTLS for SRTP keying, as DTLS server      \n"
 #endif
 
+#if defined(PJMEDIA_HAS_OPUS_CODEC) && (PJMEDIA_HAS_OPUS_CODEC != 0)
+"  --opus-pt=PT           Opus payload type                             \n"
+"  --opus-ch=CH           Opus channel count                            \n"
+"  --opus-clock-rate=CR   Opus clock rate                               \n"
+#endif
+
  "\n"
 ;
 
 
 
 
-#define THIS_FILE       "stream.c"
+#define THIS_FILE       "streamutil.c"
 
 
 
@@ -171,7 +177,7 @@ static pj_status_t create_stream( pj_pool_t *pool,
     info.tx_pt = codec_info->pt;
     info.rx_pt = codec_info->pt;
     info.ssrc = pj_rand();
-    
+
 #if PJMEDIA_HAS_RTCP_XR && PJMEDIA_STREAM_ENABLE_XR
     /* Set default RTCP XR enabled/disabled */
     info.rtcp_xr_enabled = PJ_TRUE;
@@ -390,6 +396,12 @@ int main(int argc, char *argv[])
     int tmp_key_len;
 #endif
 
+#if defined(PJMEDIA_HAS_OPUS_CODEC) && (PJMEDIA_HAS_OPUS_CODEC != 0)
+    int opus_pt = -1;
+    int opus_clock_rate = -1;
+    int opus_ch = -1;
+#endif
+
     /* Default values */
     const pjmedia_codec_info *codec_info;
     pjmedia_codec_param codec_param;
@@ -414,6 +426,11 @@ int main(int argc, char *argv[])
         OPT_RECV_ONLY   = 'i',
 #if defined(PJMEDIA_HAS_SRTP) && (PJMEDIA_HAS_SRTP != 0)
         OPT_USE_SRTP    = 'S',
+#endif
+#if defined(PJMEDIA_HAS_OPUS_CODEC) && (PJMEDIA_HAS_OPUS_CODEC != 0)
+        OPT_OPUS_PT = 'P',
+        OPT_OPUS_CH = 'C',
+        OPT_OPUS_CLOCK_RATE = 'K',
 #endif
         OPT_SRTP_TX_KEY = 'x',
         OPT_SRTP_RX_KEY = 'y',
@@ -438,6 +455,11 @@ int main(int argc, char *argv[])
         { "srtp-rx-key",    1, 0, OPT_SRTP_RX_KEY },
         { "srtp-dtls-client", 0, 0, OPT_SRTP_DTLS_CLIENT },
         { "srtp-dtls-server", 0, 0, OPT_SRTP_DTLS_SERVER },
+#endif
+#if defined(PJMEDIA_HAS_OPUS_CODEC) && (PJMEDIA_HAS_OPUS_CODEC != 0)
+        { "opus-pt", 1, 0, OPT_OPUS_PT },
+        { "opus-ch", 1, 0, OPT_OPUS_CH },
+        { "opus-clock-rate", 1, 0, OPT_OPUS_CLOCK_RATE },
 #endif
         { "help",           0, 0, OPT_HELP },
         { NULL, 0, 0, 0 },
@@ -555,6 +577,18 @@ int main(int argc, char *argv[])
             break;
 #endif
 
+#if defined(PJMEDIA_HAS_OPUS_CODEC) && (PJMEDIA_HAS_OPUS_CODEC != 0)
+        case OPT_OPUS_PT:
+            opus_pt = atoi(pj_optarg);
+            break;
+        case OPT_OPUS_CLOCK_RATE:
+            opus_clock_rate = atoi(pj_optarg);
+            break;
+        case OPT_OPUS_CH:
+            opus_ch = atoi(pj_optarg);
+            break;
+#endif
+
         case OPT_HELP:
             usage();
             return 1;
@@ -643,11 +677,30 @@ int main(int argc, char *argv[])
     if (status != PJ_SUCCESS)
         goto on_exit;
 
+#if defined(PJMEDIA_HAS_OPUS_CODEC) && (PJMEDIA_HAS_OPUS_CODEC != 0)
+    /* Opus custom settings */
+    if (!pj_stricmp2(&codec_info->encoding_name, "opus")) {
+        /* Alloc temp codec info to update settings */
+        pjmedia_codec_info *tmp = PJ_POOL_ALLOC_T(pool, pjmedia_codec_info);
+        pj_memcpy(tmp, codec_info, sizeof(pjmedia_codec_info));
+
+        if (opus_pt > 0)
+            tmp->pt = opus_pt;
+        if (opus_clock_rate > 0)
+            tmp->clock_rate = opus_clock_rate;
+        if (opus_ch > 0)
+            tmp->channel_cnt = opus_ch;
+
+        /* Use the temp codec info  */
+        codec_info = tmp;
+    }
+#endif
+
     /* Create stream based on program arguments */
-    status = create_stream(pool, med_endpt, codec_info, dir, local_port, 
+    status = create_stream(pool, med_endpt, codec_info, dir, local_port,
                            &remote_addr, mcast, &mcast_addr,
 #if defined(PJMEDIA_HAS_SRTP) && (PJMEDIA_HAS_SRTP != 0)
-                           use_srtp, &srtp_crypto_suite, 
+                           use_srtp, &srtp_crypto_suite,
                            &srtp_tx_key, &srtp_rx_key,
                            is_dtls_client, is_dtls_server,
 #endif
@@ -657,8 +710,8 @@ int main(int argc, char *argv[])
 
     /* Get codec default param for info */
     status = pjmedia_codec_mgr_get_default_param(
-                                    pjmedia_endpt_get_codec_mgr(med_endpt), 
-                                    codec_info, 
+                                    pjmedia_endpt_get_codec_mgr(med_endpt),
+                                    codec_info,
                                     &codec_param);
     /* Should be ok, as create_stream() above succeeded */
     pj_assert(status == PJ_SUCCESS);
@@ -804,8 +857,6 @@ int main(int argc, char *argv[])
 
     }
 
-
-
     /* Start deinitialization: */
 on_exit:
 
@@ -815,14 +866,9 @@ on_exit:
         PJ_ASSERT_RETURN(status == PJ_SUCCESS, 1);
     }
 
-    /* If there is master port, then we just need to destroy master port
-     * (it will recursively destroy upstream and downstream ports, which
-     * in this case are file_port and stream_port).
-     */
+    /* Destroy master port */
     if (master_port) {
-        pjmedia_master_port_destroy(master_port, PJ_TRUE);
-        play_file_port = NULL;
-        stream = NULL;
+        pjmedia_master_port_destroy(master_port, PJ_FALSE);
     }
 
     /* Destroy stream */
@@ -864,16 +910,16 @@ on_exit:
 
 
 
-static const char *good_number(char *buf, pj_int32_t val)
+static const char *good_number(char *buf, unsigned buf_size, pj_int32_t val)
 {
     if (val < 1000) {
-        pj_ansi_sprintf(buf, "%d", val);
+        pj_ansi_snprintf(buf, buf_size, "%d", val);
     } else if (val < 1000000) {
-        pj_ansi_sprintf(buf, "%d.%dK", 
+        pj_ansi_snprintf(buf, buf_size, "%d.%dK", 
                         val / 1000,
                         (val % 1000) / 100);
     } else {
-        pj_ansi_sprintf(buf, "%d.%02dM", 
+        pj_ansi_snprintf(buf, buf_size, "%d.%02dM", 
                         val / 1000000,
                         (val % 1000000) / 10000);
     }
@@ -894,9 +940,9 @@ static const char *good_number(char *buf, pj_int32_t val)
 
 #define PRINT_VOIP_MTC_VAL(s, v) \
     if (v == 127) \
-        sprintf(s, "(na)"); \
+        snprintf(s, sizeof(s), "(na)"); \
     else \
-        sprintf(s, "%d", v)
+        snprintf(s, sizeof(s), "%d", v)
 
 
 /*
@@ -911,6 +957,8 @@ static void print_stream_stat(pjmedia_stream *stream,
     pjmedia_rtcp_stat stat;
     pj_time_val now;
 
+#define SNPRINTF2(s,val)     pj_ansi_strxcpy(s, val, sizeof(s))
+#define SNPRINTF3(s,fmt,val) snprintf(s, sizeof(s), fmt, val)
 
     pj_gettimeofday(&now);
     pjmedia_stream_get_stat(stream, &stat);
@@ -920,7 +968,8 @@ static void print_stream_stat(pjmedia_stream *stream,
 
     /* Print duration */
     PJ_TIME_VAL_SUB(now, stat.start);
-    sprintf(duration, " Duration: %02ld:%02ld:%02ld.%03ld",
+    snprintf(duration, sizeof(duration),
+            " Duration: %02ld:%02ld:%02ld.%03ld",
             now.sec / 3600,
             (now.sec % 3600) / 60,
             (now.sec % 60),
@@ -930,18 +979,19 @@ static void print_stream_stat(pjmedia_stream *stream,
     printf(" Info: audio %dHz, %dms/frame, %sB/s (%sB/s +IP hdr)\n",
         PJMEDIA_PIA_SRATE(&port->info),
         PJMEDIA_PIA_PTIME(&port->info),
-        good_number(bps, (codec_param->info.avg_bps+7)/8),
-        good_number(ipbps, ((codec_param->info.avg_bps+7)/8) + 
+        good_number(bps, sizeof(bps), (codec_param->info.avg_bps+7)/8),
+        good_number(ipbps, sizeof(ipbps), ((codec_param->info.avg_bps+7)/8) + 
                            (40 * 1000 /
                             codec_param->setting.frm_per_pkt /
                             codec_param->info.frm_ptime)));
 
     if (stat.rx.update_cnt == 0)
-        strcpy(last_update, "never");
+        pj_ansi_strxcpy(last_update, "never", sizeof(last_update));
     else {
         pj_gettimeofday(&now);
         PJ_TIME_VAL_SUB(now, stat.rx.update);
-        sprintf(last_update, "%02ldh:%02ldm:%02ld.%03lds ago",
+        pj_ansi_snprintf(last_update, sizeof(last_update),
+                "%02ldh:%02ldm:%02ld.%03lds ago",
                 now.sec / 3600,
                 (now.sec % 3600) / 60,
                 now.sec % 60,
@@ -955,9 +1005,9 @@ static void print_stream_stat(pjmedia_stream *stream,
            "    loss period: %7.3f %7.3f %7.3f %7.3f %7.3f%s\n"
            "    jitter     : %7.3f %7.3f %7.3f %7.3f %7.3f%s\n",
            last_update,
-           good_number(packets, stat.rx.pkt),
-           good_number(bytes, stat.rx.bytes),
-           good_number(ipbytes, stat.rx.bytes + stat.rx.pkt * 32),
+           good_number(packets, sizeof(packets), stat.rx.pkt),
+           good_number(bytes, sizeof(bytes), stat.rx.bytes),
+           good_number(ipbytes, sizeof(ipbytes), stat.rx.bytes + stat.rx.pkt * 32),
            "",
            stat.rx.loss,
            stat.rx.loss * 100.0 / (stat.rx.pkt + stat.rx.loss),
@@ -982,11 +1032,12 @@ static void print_stream_stat(pjmedia_stream *stream,
 
 
     if (stat.tx.update_cnt == 0)
-        strcpy(last_update, "never");
+        pj_ansi_strxcpy(last_update, "never", sizeof(last_update));
     else {
         pj_gettimeofday(&now);
         PJ_TIME_VAL_SUB(now, stat.tx.update);
-        sprintf(last_update, "%02ldh:%02ldm:%02ld.%03lds ago",
+        pj_ansi_snprintf(last_update, sizeof(last_update),
+                "%02ldh:%02ldm:%02ld.%03lds ago",
                 now.sec / 3600,
                 (now.sec % 3600) / 60,
                 now.sec % 60,
@@ -1000,9 +1051,9 @@ static void print_stream_stat(pjmedia_stream *stream,
            "    loss period: %7.3f %7.3f %7.3f %7.3f %7.3f%s\n"
            "    jitter     : %7.3f %7.3f %7.3f %7.3f %7.3f%s\n",
            last_update,
-           good_number(packets, stat.tx.pkt),
-           good_number(bytes, stat.tx.bytes),
-           good_number(ipbytes, stat.tx.bytes + stat.tx.pkt * 32),
+           good_number(packets, sizeof(packets), stat.tx.pkt),
+           good_number(bytes, sizeof(bytes), stat.tx.bytes),
+           good_number(ipbytes, sizeof(ipbytes), stat.tx.bytes + stat.tx.pkt * 32),
            "",
            stat.tx.loss,
            stat.tx.loss * 100.0 / (stat.tx.pkt + stat.tx.loss),
@@ -1055,14 +1106,14 @@ static void print_stream_stat(pjmedia_stream *stream,
         puts(" Statistics Summary");
 
         if (xr_stat.rx.stat_sum.l)
-            sprintf(loss, "%d", xr_stat.rx.stat_sum.lost);
+            SNPRINTF3(loss, "%d", xr_stat.rx.stat_sum.lost);
         else
-            sprintf(loss, "(na)");
+            SNPRINTF2(loss, "(na)");
 
         if (xr_stat.rx.stat_sum.d)
-            sprintf(dup, "%d", xr_stat.rx.stat_sum.dup);
+            SNPRINTF3(dup, "%d", xr_stat.rx.stat_sum.dup);
         else
-            sprintf(dup, "(na)");
+            SNPRINTF2(dup, "(na)");
 
         if (xr_stat.rx.stat_sum.j) {
             unsigned jmin, jmax, jmean, jdev;
@@ -1076,26 +1127,27 @@ static void print_stream_stat(pjmedia_stream *stream,
             SAMPLES_TO_USEC(jdev, 
                            pj_math_stat_get_stddev(&xr_stat.rx.stat_sum.jitter),
                            port->info.fmt.det.aud.clock_rate);
-            sprintf(jitter, "%7.3f %7.3f %7.3f %7.3f", 
+            snprintf(jitter, sizeof(jitter), "%7.3f %7.3f %7.3f %7.3f",
                     jmin/1000.0, jmean/1000.0, jmax/1000.0, jdev/1000.0);
         } else
-            sprintf(jitter, "(report not available)");
+            SNPRINTF2(jitter, "(report not available)");
 
         if (xr_stat.rx.stat_sum.t) {
-            sprintf(toh, "%11d %11d %11d %11d", 
+            snprintf(toh, sizeof(toh), "%11d %11d %11d %11d",
                     xr_stat.rx.stat_sum.toh.min,
                     xr_stat.rx.stat_sum.toh.mean,
                     xr_stat.rx.stat_sum.toh.max,
                     pj_math_stat_get_stddev(&xr_stat.rx.stat_sum.toh));
         } else
-            sprintf(toh, "(report not available)");
+            SNPRINTF2(toh, "(report not available)");
 
         if (xr_stat.rx.stat_sum.update.sec == 0)
-            strcpy(last_update, "never");
+            pj_ansi_strxcpy(last_update, "never", sizeof(last_update));
         else {
             pj_gettimeofday(&now);
             PJ_TIME_VAL_SUB(now, xr_stat.rx.stat_sum.update);
-            sprintf(last_update, "%02ldh:%02ldm:%02ld.%03lds ago",
+            pj_ansi_snprintf(last_update, sizeof(last_update),
+                    "%02ldh:%02ldm:%02ld.%03lds ago",
                     now.sec / 3600,
                     (now.sec % 3600) / 60,
                     now.sec % 60,
@@ -1118,14 +1170,14 @@ static void print_stream_stat(pjmedia_stream *stream,
                );
 
         if (xr_stat.tx.stat_sum.l)
-            sprintf(loss, "%d", xr_stat.tx.stat_sum.lost);
+            SNPRINTF3(loss, "%d", xr_stat.tx.stat_sum.lost);
         else
-            sprintf(loss, "(na)");
+            SNPRINTF2(loss, "(na)");
 
         if (xr_stat.tx.stat_sum.d)
-            sprintf(dup, "%d", xr_stat.tx.stat_sum.dup);
+            SNPRINTF3(dup, "%d", xr_stat.tx.stat_sum.dup);
         else
-            sprintf(dup, "(na)");
+            SNPRINTF2(dup, "(na)");
 
         if (xr_stat.tx.stat_sum.j) {
             unsigned jmin, jmax, jmean, jdev;
@@ -1139,26 +1191,27 @@ static void print_stream_stat(pjmedia_stream *stream,
             SAMPLES_TO_USEC(jdev, 
                            pj_math_stat_get_stddev(&xr_stat.tx.stat_sum.jitter),
                            port->info.fmt.det.aud.clock_rate);
-            sprintf(jitter, "%7.3f %7.3f %7.3f %7.3f", 
+            snprintf(jitter, sizeof(jitter), "%7.3f %7.3f %7.3f %7.3f",
                     jmin/1000.0, jmean/1000.0, jmax/1000.0, jdev/1000.0);
         } else
-            sprintf(jitter, "(report not available)");
+            SNPRINTF2(jitter, "(report not available)");
 
         if (xr_stat.tx.stat_sum.t) {
-            sprintf(toh, "%11d %11d %11d %11d", 
+            snprintf(toh, sizeof(toh), "%11d %11d %11d %11d",
                     xr_stat.tx.stat_sum.toh.min,
                     xr_stat.tx.stat_sum.toh.mean,
                     xr_stat.tx.stat_sum.toh.max,
                     pj_math_stat_get_stddev(&xr_stat.rx.stat_sum.toh));
         } else
-            sprintf(toh,    "(report not available)");
+            SNPRINTF2(toh,    "(report not available)");
 
         if (xr_stat.tx.stat_sum.update.sec == 0)
-            strcpy(last_update, "never");
+            pj_ansi_strxcpy(last_update, "never", sizeof(last_update));
         else {
             pj_gettimeofday(&now);
             PJ_TIME_VAL_SUB(now, xr_stat.tx.stat_sum.update);
-            sprintf(last_update, "%02ldh:%02ldm:%02ld.%03lds ago",
+            pj_ansi_snprintf(last_update, sizeof(last_update),
+                    "%02ldh:%02ldm:%02ld.%03lds ago",
                     now.sec / 3600,
                     (now.sec % 3600) / 60,
                     now.sec % 60,
@@ -1193,40 +1246,41 @@ static void print_stream_stat(pjmedia_stream *stream,
 
         switch ((xr_stat.rx.voip_mtc.rx_config>>6) & 3) {
             case PJMEDIA_RTCP_XR_PLC_DIS:
-                sprintf(plc, "DISABLED");
+                SNPRINTF2(plc, "DISABLED");
                 break;
             case PJMEDIA_RTCP_XR_PLC_ENH:
-                sprintf(plc, "ENHANCED");
+                SNPRINTF2(plc, "ENHANCED");
                 break;
             case PJMEDIA_RTCP_XR_PLC_STD:
-                sprintf(plc, "STANDARD");
+                SNPRINTF2(plc, "STANDARD");
                 break;
             case PJMEDIA_RTCP_XR_PLC_UNK:
             default:
-                sprintf(plc, "UNKNOWN");
+                SNPRINTF2(plc, "UNKNOWN");
                 break;
         }
 
         switch ((xr_stat.rx.voip_mtc.rx_config>>4) & 3) {
             case PJMEDIA_RTCP_XR_JB_FIXED:
-                sprintf(jba, "FIXED");
+                SNPRINTF2(jba, "FIXED");
                 break;
             case PJMEDIA_RTCP_XR_JB_ADAPTIVE:
-                sprintf(jba, "ADAPTIVE");
+                SNPRINTF2(jba, "ADAPTIVE");
                 break;
             default:
-                sprintf(jba, "UNKNOWN");
+                SNPRINTF2(jba, "UNKNOWN");
                 break;
         }
 
-        sprintf(jbr, "%d", xr_stat.rx.voip_mtc.rx_config & 0x0F);
+        SNPRINTF3(jbr, "%d", xr_stat.rx.voip_mtc.rx_config & 0x0F);
 
         if (xr_stat.rx.voip_mtc.update.sec == 0)
-            strcpy(last_update, "never");
+            pj_ansi_strxcpy(last_update, "never", sizeof(last_update));
         else {
             pj_gettimeofday(&now);
             PJ_TIME_VAL_SUB(now, xr_stat.rx.voip_mtc.update);
-            sprintf(last_update, "%02ldh:%02ldm:%02ld.%03lds ago",
+            pj_ansi_snprintf(last_update, sizeof(last_update),
+                    "%02ldh:%02ldm:%02ld.%03lds ago",
                     now.sec / 3600,
                     (now.sec % 3600) / 60,
                     now.sec % 60,
@@ -1280,40 +1334,41 @@ static void print_stream_stat(pjmedia_stream *stream,
 
         switch ((xr_stat.tx.voip_mtc.rx_config>>6) & 3) {
             case PJMEDIA_RTCP_XR_PLC_DIS:
-                sprintf(plc, "DISABLED");
+                SNPRINTF2(plc, "DISABLED");
                 break;
             case PJMEDIA_RTCP_XR_PLC_ENH:
-                sprintf(plc, "ENHANCED");
+                SNPRINTF2(plc, "ENHANCED");
                 break;
             case PJMEDIA_RTCP_XR_PLC_STD:
-                sprintf(plc, "STANDARD");
+                SNPRINTF2(plc, "STANDARD");
                 break;
             case PJMEDIA_RTCP_XR_PLC_UNK:
             default:
-                sprintf(plc, "unknown");
+                SNPRINTF2(plc, "unknown");
                 break;
         }
 
         switch ((xr_stat.tx.voip_mtc.rx_config>>4) & 3) {
             case PJMEDIA_RTCP_XR_JB_FIXED:
-                sprintf(jba, "FIXED");
+                SNPRINTF2(jba, "FIXED");
                 break;
             case PJMEDIA_RTCP_XR_JB_ADAPTIVE:
-                sprintf(jba, "ADAPTIVE");
+                SNPRINTF2(jba, "ADAPTIVE");
                 break;
             default:
-                sprintf(jba, "unknown");
+                SNPRINTF2(jba, "unknown");
                 break;
         }
 
-        sprintf(jbr, "%d", xr_stat.tx.voip_mtc.rx_config & 0x0F);
+        SNPRINTF3(jbr, "%d", xr_stat.tx.voip_mtc.rx_config & 0x0F);
 
         if (xr_stat.tx.voip_mtc.update.sec == 0)
-            strcpy(last_update, "never");
+            pj_ansi_strxcpy(last_update, "never", sizeof(last_update));
         else {
             pj_gettimeofday(&now);
             PJ_TIME_VAL_SUB(now, xr_stat.tx.voip_mtc.update);
-            sprintf(last_update, "%02ldh:%02ldm:%02ld.%03lds ago",
+            pj_ansi_snprintf(last_update, sizeof(last_update),
+                    "%02ldh:%02ldm:%02ld.%03lds ago",
                     now.sec / 3600,
                     (now.sec % 3600) / 60,
                     now.sec % 60,
